@@ -1,4 +1,4 @@
-# $Id: __init__.py 8267 2019-07-20 19:50:51Z grubert $
+# $Id: __init__.py 8864 2021-10-26 11:46:55Z grubert $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -56,23 +56,62 @@ from collections import namedtuple
 
 __docformat__ = 'reStructuredText'
 
-__version__ = '0.15'
+__version__ = '0.18'
 """Docutils version identifier (complies with PEP 440)::
 
     major.minor[.micro][releaselevel[serial]][.dev]
 
-For version comparison operations, use `__version_info__` (which see, below)
+For version comparison operations, use `__version_info__` (see, below)
 rather than parsing the text of `__version__`.
 
 See 'Version Numbering' in docs/dev/policies.txt.
 """
 
-VersionInfo = namedtuple(
-    'VersionInfo', 'major minor micro releaselevel serial release')
+# from functools import total_ordering
+# @total_ordering
+class VersionInfo(namedtuple('VersionInfo',
+                             'major minor micro releaselevel serial release')):
+
+    def __new__(cls, major=0, minor=0, micro=0,
+                releaselevel='final', serial=0, release=True):
+        releaselevels = ('alpha', 'beta', 'candidate', 'final')
+        if releaselevel not in releaselevels:
+            raise ValueError('releaselevel must be one of %r.'
+                             % (releaselevels, ))
+        if releaselevel == 'final':
+            if not release:
+                raise ValueError('releaselevel "final" must not be used '
+                             'with development versions (leads to wrong '
+                             'version ordering of the related __version__')
+            if serial != 0:
+                raise ValueError('"serial" must be 0 for final releases')
+
+        return super(VersionInfo, cls).__new__(cls, major, minor, micro,
+                                               releaselevel, serial, release)
+
+    def __lt__(self, other):
+        if isinstance(other, tuple):
+            other = VersionInfo(*other)
+        return tuple.__lt__(self, other)
+
+    def __gt__(self, other):
+        if isinstance(other, tuple):
+            other = VersionInfo(*other)
+        return tuple.__gt__(self, other)
+
+    def __le__(self, other):
+        if isinstance(other, tuple):
+            other = VersionInfo(*other)
+        return tuple.__le__(self, other)
+
+    def __ge__(self, other):
+        if isinstance(other, tuple):
+            other = VersionInfo(*other)
+        return tuple.__ge__(self, other)
 
 __version_info__ = VersionInfo(
     major=0,
-    minor=15,
+    minor=18,
     micro=0,
     releaselevel='final', # one of 'alpha', 'beta', 'candidate', 'final'
     # pre-release serial number (0 for final releases and active development):
@@ -88,11 +127,12 @@ __version_details__ = 'release'
 """
 
 
-class ApplicationError(StandardError): pass
+class ApplicationError(Exception): pass
+
 class DataError(ApplicationError): pass
 
 
-class SettingsSpec:
+class SettingsSpec(object):
 
     """
     Runtime setting specification base class.
@@ -190,7 +230,7 @@ class TransformSpec:
     """List of functions to try to resolve unknown references.  Unknown
     references have a 'refname' attribute which doesn't correspond to any
     target in the document.  Called when the transforms in
-    `docutils.tranforms.references` are unable to find a correct target.  The
+    `docutils.transforms.references` are unable to find a correct target.  The
     list should contain functions which will try to resolve unknown
     references, with the following signature::
 
