@@ -1,4 +1,4 @@
-# $Id: frontend.py 8583 2020-12-01 10:53:27Z milde $
+# $Id: frontend.py 8858 2021-10-19 17:03:02Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -333,6 +333,13 @@ class Values(optparse.Values):
         """Return a shallow copy of `self`."""
         return self.__class__(defaults=self.__dict__)
 
+    def setdefault(self, name, default):
+        """V.setdefault(n[,d]) -> getattr(V,n,d), also set D.n=d if n not in D or None.
+        """
+        if getattr(self, name, None) is None:
+            setattr(self, name, default)
+        return getattr(self, name)
+
 
 class Option(optparse.Option):
 
@@ -549,7 +556,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           ['--help', '-h'], {'action': 'help'}),
          # Typically not useful for non-programmatical use:
          (SUPPRESS_HELP, ['--id-prefix'], {'default': ''}),
-         (SUPPRESS_HELP, ['--auto-id-prefix'], {'default': 'id'}),
+         (SUPPRESS_HELP, ['--auto-id-prefix'], {'default': '%'}),
          # Hidden options, for development use only:
          (SUPPRESS_HELP, ['--dump-settings'], {'action': 'store_true'}),
          (SUPPRESS_HELP, ['--dump-internals'], {'action': 'store_true'}),
@@ -637,7 +644,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                     option = group.add_option(help=help_text, *option_strings,
                                               **kwargs)
                     if kwargs.get('action') == 'append':
-                        self.lists[option.dest] = 1
+                        self.lists[option.dest] = True
                 if component.settings_defaults:
                     self.defaults.update(component.settings_defaults)
         for component in components:
@@ -676,7 +683,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         parser.read(config_file, self)
         self.config_files.extend(parser._files)
         base_path = os.path.dirname(config_file)
-        applied = {}
+        applied = set()
         settings = Values()
         for component in self.components:
             if not component:
@@ -685,7 +692,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                             + (component.config_section,)):
                 if section in applied:
                     continue
-                applied[section] = 1
+                applied.add(section)
                 settings.update(parser.get_section(section), self)
         make_paths_absolute(
             settings.__dict__, self.relative_path_settings, base_path)
@@ -776,7 +783,7 @@ Skipping "%s" configuration file.
             filenames = [filenames]
         for filename in filenames:
             try:
-                # Config files must be UTF-8-encoded:
+                # Config files are UTF-8-encoded:
                 fp = codecs.open(filename, 'r', 'utf-8')
             except IOError:
                 continue
