@@ -1,4 +1,4 @@
- # $Id: statemachine.py 7037 2011-05-19 08:56:27Z milde $
+ # $Id: statemachine.py 8860 2021-10-22 16:39:59Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -103,16 +103,21 @@ How To Use This Module
 
        sm.unlink()
 """
+from __future__ import print_function
 
 __docformat__ = 'restructuredtext'
 
 import sys
 import re
-import types
 import unicodedata
-from docutils.error_reporting import ErrorOutput
+from docutils import utils
+from docutils.utils.error_reporting import ErrorOutput
 
-class StateMachine:
+if sys.version_info >= (3, 0):
+    unicode = str  # noqa
+
+
+class StateMachine(object):
 
     """
     A finite state machine for text filters using regular expressions.
@@ -126,7 +131,7 @@ class StateMachine:
     results of processing in a list.
     """
 
-    def __init__(self, state_classes, initial_state, debug=0):
+    def __init__(self, state_classes, initial_state, debug=False):
         """
         Initialize a `StateMachine` object; add state objects.
 
@@ -211,50 +216,46 @@ class StateMachine:
         self.line_offset = -1
         self.current_state = initial_state or self.initial_state
         if self.debug:
-            print >>self._stderr, (
-                u'\nStateMachine.run: input_lines (line_offset=%s):\n| %s'
-                % (self.line_offset, u'\n| '.join(self.input_lines)))
+            print(u'\nStateMachine.run: input_lines (line_offset=%s):\n| %s'
+                  % (self.line_offset, u'\n| '.join(self.input_lines)), file=self._stderr)
         transitions = None
         results = []
         state = self.get_state()
         try:
             if self.debug:
-                print >>self._stderr, '\nStateMachine.run: bof transition'
+                print('\nStateMachine.run: bof transition', file=self._stderr)
             context, result = state.bof(context)
             results.extend(result)
-            while 1:
+            while True:
                 try:
                     try:
                         self.next_line()
                         if self.debug:
                             source, offset = self.input_lines.info(
                                 self.line_offset)
-                            print >>self._stderr, (
-                                u'\nStateMachine.run: line (source=%r, '
-                                u'offset=%r):\n| %s'
-                                % (source, offset, self.line))
+                            print(u'\nStateMachine.run: line (source=%r, '
+                                  u'offset=%r):\n| %s'
+                                  % (source, offset, self.line), file=self._stderr)
                         context, next_state, result = self.check_line(
                             context, state, transitions)
                     except EOFError:
                         if self.debug:
-                            print >>self._stderr, (
-                                '\nStateMachine.run: %s.eof transition'
-                                % state.__class__.__name__)
+                            print('\nStateMachine.run: %s.eof transition'
+                                  % state.__class__.__name__, file=self._stderr)
                         result = state.eof(context)
                         results.extend(result)
                         break
                     else:
                         results.extend(result)
-                except TransitionCorrection, exception:
+                except TransitionCorrection as exception:
                     self.previous_line() # back up for another try
                     transitions = (exception.args[0],)
                     if self.debug:
-                        print >>self._stderr, (
-                              '\nStateMachine.run: TransitionCorrection to '
+                        print('\nStateMachine.run: TransitionCorrection to '
                               'state "%s", transition %s.'
-                              % (state.__class__.__name__, transitions[0]))
+                              % (state.__class__.__name__, transitions[0]), file=self._stderr)
                     continue
-                except StateCorrection, exception:
+                except StateCorrection as exception:
                     self.previous_line() # back up for another try
                     next_state = exception.args[0]
                     if len(exception.args) == 1:
@@ -262,10 +263,9 @@ class StateMachine:
                     else:
                         transitions = (exception.args[1],)
                     if self.debug:
-                        print >>self._stderr, (
-                              '\nStateMachine.run: StateCorrection to state '
+                        print('\nStateMachine.run: StateCorrection to state '
                               '"%s", transition %s.'
-                              % (next_state, transitions[0]))
+                              % (next_state, transitions[0]), file=self._stderr)
                 else:
                     transitions = None
                 state = self.get_state(next_state)
@@ -286,11 +286,10 @@ class StateMachine:
         """
         if next_state:
             if self.debug and next_state != self.current_state:
-                print >>self._stderr, (
-                    '\nStateMachine.get_state: Changing state from '
-                    '"%s" to "%s" (input line %s).'
-                    % (self.current_state, next_state,
-                       self.abs_line_number()))
+                print('\nStateMachine.get_state: Changing state from '
+                      '"%s" to "%s" (input line %s).'
+                      % (self.current_state, next_state,
+                         self.abs_line_number()), file=self._stderr)
             self.current_state = next_state
         try:
             return self.states[self.current_state]
@@ -311,7 +310,7 @@ class StateMachine:
             self.notify_observers()
 
     def is_next_line_blank(self):
-        """Return 1 if the next line is blank or non-existant."""
+        """Return True if the next line is blank or non-existent."""
         try:
             return not self.input_lines[self.line_offset + 1].strip()
         except IndexError:
@@ -380,15 +379,11 @@ class StateMachine:
             # line is None if index is "Just past the end"
             src, srcline = self.get_source_and_line(offset + self.input_offset)
             return src, srcline + 1
-        except (IndexError): # `offset` is off the list
+        except (IndexError):  # `offset` is off the list
             src, srcline = None, None
             # raise AssertionError('cannot find line %d in %s lines' %
             #                      (offset, len(self.input_lines)))
             #                      # list(self.input_lines.lines())))
-        # assert offset == srcoffset, str(self.input_lines)
-        # print "get_source_and_line(%s):" % lineno,
-        # print offset + 1, '->', src, srcline
-        # print self.input_lines
         return (src, srcline)
 
     def insert_input(self, input_lines, source):
@@ -401,7 +396,7 @@ class StateMachine:
         self.input_lines.insert(self.line_offset + 2,
                                 StringList(input_lines, source))
 
-    def get_text_block(self, flush_left=0):
+    def get_text_block(self, flush_left=False):
         """
         Return a contiguous block of text.
 
@@ -414,8 +409,8 @@ class StateMachine:
                                                     flush_left)
             self.next_line(len(block) - 1)
             return block
-        except UnexpectedIndentationError, error:
-            block, source, lineno = error.args
+        except UnexpectedIndentationError as err:
+            block = err.args[0]
             self.next_line(len(block) - 1) # advance to last line of block
             raise
 
@@ -443,24 +438,21 @@ class StateMachine:
             transitions =  state.transition_order
         state_correction = None
         if self.debug:
-            print >>self._stderr, (
-                  '\nStateMachine.check_line: state="%s", transitions=%r.'
-                  % (state.__class__.__name__, transitions))
+            print('\nStateMachine.check_line: state="%s", transitions=%r.'
+                  % (state.__class__.__name__, transitions), file=self._stderr)
         for name in transitions:
             pattern, method, next_state = state.transitions[name]
             match = pattern.match(self.line)
             if match:
                 if self.debug:
-                    print >>self._stderr, (
-                          '\nStateMachine.check_line: Matched transition '
+                    print('\nStateMachine.check_line: Matched transition '
                           '"%s" in state "%s".'
-                          % (name, state.__class__.__name__))
+                          % (name, state.__class__.__name__), file=self._stderr)
                 return method(match, context, next_state)
         else:
             if self.debug:
-                print >>self._stderr, (
-                      '\nStateMachine.check_line: No match in state "%s".'
-                      % state.__class__.__name__)
+                print('\nStateMachine.check_line: No match in state "%s".'
+                      % state.__class__.__name__, file=self._stderr)
             return state.no_match(context, transitions)
 
     def add_state(self, state_class):
@@ -492,10 +484,10 @@ class StateMachine:
     def error(self):
         """Report error details."""
         type, value, module, line, function = _exception_data()
-        print >>self._stderr, u'%s: %s' % (type, value)
-        print >>self._stderr, 'input line %s' % (self.abs_line_number())
-        print >>self._stderr, (u'module %s, line %s, function %s' %
-                               (module, line, function))
+        print(u'%s: %s' % (type, value), file=self._stderr)
+        print('input line %s' % (self.abs_line_number()), file=self._stderr)
+        print((u'module %s, line %s, function %s' %
+                               (module, line, function)), file=self._stderr)
 
     def attach_observer(self, observer):
         """
@@ -516,7 +508,7 @@ class StateMachine:
             observer(*info)
 
 
-class State:
+class State(object):
 
     """
     State superclass. Contains a list of transitions, and transition methods.
@@ -528,7 +520,7 @@ class State:
       ``match.end()`` gives the end index.
     - A context object, whose meaning is application-defined (initial value
       ``None``). It can be used to store any information required by the state
-      machine, and the retured context is passed on to the next transition
+      machine, and the returned context is passed on to the next transition
       method unchanged.
     - The name of the next state, a string, taken from the transitions list;
       normally it is returned unchanged, but it may be altered by the
@@ -593,14 +585,14 @@ class State:
     defaults.
     """
 
-    def __init__(self, state_machine, debug=0):
+    def __init__(self, state_machine, debug=False):
         """
         Initialize a `State` object; make & add initial transitions.
 
         Parameters:
 
         - `statemachine`: the controlling `StateMachine` object.
-        - `debug`: a boolean; produce verbose output if true (nonzero).
+        - `debug`: a boolean; produce verbose output if true.
         """
 
         self.transition_order = []
@@ -713,7 +705,7 @@ class State:
         try:
             pattern = self.patterns[name]
             if not hasattr(pattern, 'match'):
-                pattern = re.compile(pattern)
+                pattern = self.patterns[name] = re.compile(pattern)
         except KeyError:
             raise TransitionPatternNotFound(
                   '%s.patterns[%r]' % (self.__class__.__name__, name))
@@ -736,7 +728,7 @@ class State:
         names = []
         transitions = {}
         for namestate in name_list:
-            if type(namestate) is stringtype:
+            if isinstance(namestate, stringtype):
                 transitions[namestate] = self.make_transition(namestate)
                 names.append(namestate)
             else:
@@ -801,17 +793,15 @@ class StateMachineWS(StateMachine):
       known.
     """
 
-    def get_indented(self, until_blank=0, strip_indent=1):
+    def get_indented(self, until_blank=False, strip_indent=True):
         """
         Return a block of indented lines of text, and info.
 
         Extract an indented block where the indent is unknown for all lines.
 
         :Parameters:
-            - `until_blank`: Stop collecting at the first blank line if true
-              (1).
-            - `strip_indent`: Strip common leading indent if true (1,
-              default).
+            - `until_blank`: Stop collecting at the first blank line if true.
+            - `strip_indent`: Strip common leading indent if true (default).
 
         :Return:
             - the indented block (a list of lines of text),
@@ -829,7 +819,7 @@ class StateMachineWS(StateMachine):
             offset += 1
         return indented, indent, offset, blank_finish
 
-    def get_known_indented(self, indent, until_blank=0, strip_indent=1):
+    def get_known_indented(self, indent, until_blank=False, strip_indent=True):
         """
         Return an indented block and info.
 
@@ -840,10 +830,9 @@ class StateMachineWS(StateMachine):
 
         :Parameters:
             - `indent`: The number of indent columns/characters.
-            - `until_blank`: Stop collecting at the first blank line if true
-              (1).
+            - `until_blank`: Stop collecting at the first blank line if true.
             - `strip_indent`: Strip `indent` characters of indentation if true
-              (1, default).
+              (default).
 
         :Return:
             - the indented block,
@@ -860,8 +849,8 @@ class StateMachineWS(StateMachine):
             offset += 1
         return indented, offset, blank_finish
 
-    def get_first_known_indented(self, indent, until_blank=0, strip_indent=1,
-                                 strip_top=1):
+    def get_first_known_indented(self, indent, until_blank=False,
+                                 strip_indent=True, strip_top=True):
         """
         Return an indented block and info.
 
@@ -946,8 +935,8 @@ class StateWS(State):
     `indent_sm_kwargs`. Override it in subclasses to avoid the default.
     """
 
-    ws_patterns = {'blank': ' *$',
-                   'indent': ' +'}
+    ws_patterns = {'blank': re.compile(' *$'),
+                   'indent': re.compile(' +')}
     """Patterns for default whitespace transitions.  May be overridden in
     subclasses."""
 
@@ -955,7 +944,7 @@ class StateWS(State):
     """Default initial whitespace transitions, added before those listed in
     `State.initial_transitions`.  May be overridden in subclasses."""
 
-    def __init__(self, state_machine, debug=0):
+    def __init__(self, state_machine, debug=False):
         """
         Initialize a `StateSM` object; extends `State.__init__()`.
 
@@ -1035,7 +1024,7 @@ class StateWS(State):
         return context, next_state, results
 
 
-class _SearchOverride:
+class _SearchOverride(object):
 
     """
     Mix-in class to override `StateMachine` regular expression behavior.
@@ -1068,7 +1057,7 @@ class SearchStateMachineWS(_SearchOverride, StateMachineWS):
     pass
 
 
-class ViewList:
+class ViewList(object):
 
     """
     List with extended functionality: slices of ViewList objects are child
@@ -1128,7 +1117,12 @@ class ViewList:
     def __ne__(self, other): return self.data != self.__cast(other)
     def __gt__(self, other): return self.data >  self.__cast(other)
     def __ge__(self, other): return self.data >= self.__cast(other)
-    def __cmp__(self, other): return cmp(self.data, self.__cast(other))
+
+    def __cmp__(self, other):
+        # from https://docs.python.org/3.0/whatsnew/3.0.html
+        mine = self.data
+        yours = self.__cast(other)
+        return (mine > yours) - (yours < mine)
 
     def __cast(self, other):
         if isinstance(other, ViewList):
@@ -1144,7 +1138,7 @@ class ViewList:
     # just works.
 
     def __getitem__(self, i):
-        if isinstance(i, types.SliceType):
+        if isinstance(i, slice):
             assert i.step in (None, 1),  'cannot handle slice with stride'
             return self.__class__(self.data[i.start:i.stop],
                                   items=self.items[i.start:i.stop],
@@ -1153,7 +1147,7 @@ class ViewList:
             return self.data[i]
 
     def __setitem__(self, i, item):
-        if isinstance(i, types.SliceType):
+        if isinstance(i, slice):
             assert i.step in (None, 1), 'cannot handle slice with stride'
             if not isinstance(item, ViewList):
                 raise TypeError('assigning non-ViewList to ViewList slice')
@@ -1294,8 +1288,7 @@ class ViewList:
         self.parent = None
 
     def sort(self, *args):
-        tmp = zip(self.data, self.items)
-        tmp.sort(*args)
+        tmp = sorted(zip(self.data, self.items), *args)
         self.data = [entry[0] for entry in tmp]
         self.items = [entry[1] for entry in tmp]
         self.parent = None
@@ -1330,14 +1323,14 @@ class ViewList:
     def pprint(self):
         """Print the list in `grep` format (`source:offset:value` lines)"""
         for line in self.xitems():
-            print "%s:%d:%s" % line
+            print("%s:%d:%s" % line)
 
 
 class StringList(ViewList):
 
     """A `ViewList` with string-specific methods."""
 
-    def trim_left(self, length, start=0, end=sys.maxint):
+    def trim_left(self, length, start=0, end=sys.maxsize):
         """
         Trim `length` characters off the beginning of each item, in-place,
         from index `start` to `end`.  No whitespace-checking is done on the
@@ -1346,7 +1339,7 @@ class StringList(ViewList):
         self.data[start:end] = [line[length:]
                                 for line in self.data[start:end]]
 
-    def get_text_block(self, start, flush_left=0):
+    def get_text_block(self, start, flush_left=False):
         """
         Return a contiguous block of text.
 
@@ -1367,7 +1360,7 @@ class StringList(ViewList):
             end += 1
         return self[start:end]
 
-    def get_indented(self, start=0, until_blank=0, strip_indent=1,
+    def get_indented(self, start=0, until_blank=False, strip_indent=True,
                      block_indent=None, first_indent=None):
         """
         Extract and return a StringList of indented lines of text.
@@ -1385,7 +1378,7 @@ class StringList(ViewList):
           - `first_indent`: The indent of the first line, if known.
 
         :Return:
-          - a StringList of indented lines with mininum indent removed;
+          - a StringList of indented lines with minimum indent removed;
           - the amount of the indent;
           - a boolean: did the indented block finish with a blank line or EOF?
         """
@@ -1427,10 +1420,20 @@ class StringList(ViewList):
             block.trim_left(indent, start=(first_indent is not None))
         return block, indent or 0, blank_finish
 
-    def get_2D_block(self, top, left, bottom, right, strip_indent=1):
+    def get_2D_block(self, top, left, bottom, right, strip_indent=True):
         block = self[top:bottom]
         indent = right
         for i in range(len(block.data)):
+            # get slice from line, care for combining characters
+            ci = utils.column_indices(block.data[i])
+            try:
+                left = ci[left]
+            except IndexError:
+                left += len(block.data[i]) - len(ci)
+            try:
+                right = ci[right]
+            except IndexError:
+                right += len(block.data[i]) - len(ci)
             block.data[i] = line = block.data[i][left:right].rstrip()
             if line:
                 indent = min(indent, len(line) - len(line.lstrip()))
@@ -1443,10 +1446,7 @@ class StringList(ViewList):
         Pad all double-width characters in self by appending `pad_char` to each.
         For East Asian language support.
         """
-        if hasattr(unicodedata, 'east_asian_width'):
-            east_asian_width = unicodedata.east_asian_width
-        else:
-            return                      # new in Python 2.4
+        east_asian_width = unicodedata.east_asian_width
         for i in range(len(self.data)):
             line = self.data[i]
             if isinstance(line, unicode):
@@ -1491,8 +1491,7 @@ class StateCorrection(Exception):
     transition name.
     """
 
-
-def string2lines(astring, tab_width=8, convert_whitespace=0,
+def string2lines(astring, tab_width=8, convert_whitespace=False,
                  whitespace=re.compile('[\v\f]')):
     """
     Return a list of one-line strings with tabs expanded, no newlines, and
@@ -1506,10 +1505,13 @@ def string2lines(astring, tab_width=8, convert_whitespace=0,
     - `astring`: a multi-line string.
     - `tab_width`: the number of columns between tab stops.
     - `convert_whitespace`: convert form feeds and vertical tabs to spaces?
+    - `whitespace`: pattern object with the to-be-converted
+      whitespace characters (default [\\v\\f]).
     """
     if convert_whitespace:
         astring = whitespace.sub(' ', astring)
-    return [s.expandtabs(tab_width).rstrip() for s in astring.splitlines()]
+    lines = [s.expandtabs(tab_width).rstrip() for s in astring.splitlines()]
+    return lines
 
 def _exception_data():
     """
