@@ -1,5 +1,5 @@
 # coding: utf-8
-# $Id: __init__.py 8231 2018-11-20 23:55:14Z milde $
+# $Id: __init__.py 8860 2021-10-22 16:39:59Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -18,9 +18,13 @@ import warnings
 import unicodedata
 from docutils import ApplicationError, DataError, __version_info__
 from docutils import nodes
+from docutils.nodes import unescape
 import docutils.io
 from docutils.utils.error_reporting import ErrorOutput, SafeString
 
+if sys.version_info >= (3, 0):
+    unicode = str
+
 
 class SystemMessage(ApplicationError):
 
@@ -32,7 +36,7 @@ class SystemMessage(ApplicationError):
 class SystemMessagePropagation(ApplicationError): pass
 
 
-class Reporter:
+class Reporter(object):
 
     """
     Info/warning/error reporter and ``system_message`` element generator.
@@ -172,7 +176,6 @@ class Reporter:
         if not 'source' in attributes: # 'line' is absolute line number
             try: # look up (source, line-in-source)
                 source, line = self.get_source_and_line(attributes.get('line'))
-                # print "locator lookup", kwargs.get('line'), "->", source, line
             except AttributeError:
                 source, line = None, None
             if source is not None:
@@ -326,7 +329,7 @@ def assemble_option_dict(option_list, options_spec):
             raise DuplicateOptionError('duplicate option "%s"' % name)
         try:
             options[name] = convertor(value)
-        except (ValueError, TypeError), detail:
+        except (ValueError, TypeError) as detail:
             raise detail.__class__('(option: "%s"; value: %r)\n%s'
                                    % (name, value, ' '.join(detail.args)))
     return options
@@ -339,7 +342,7 @@ def decode_path(path):
     """
     Ensure `path` is Unicode. Return `nodes.reprunicode` object.
 
-    Decode file/path string in a failsave manner if not already done.
+    Decode file/path string in a failsafe manner if not already done.
     """
     # see also http://article.gmane.org/gmane.text.docutils.user/2905
     if isinstance(path, unicode):
@@ -347,7 +350,10 @@ def decode_path(path):
     try:
         path = path.decode(sys.getfilesystemencoding(), 'strict')
     except AttributeError: # default value None has no decode method
-        return nodes.reprunicode(path)
+        if not path:
+            return nodes.reprunicode('')
+        raise ValueError('`path` value must be a String or ``None``, not %r'
+                         %path)
     except UnicodeDecodeError:
         try:
             path = path.decode('utf-8', 'strict')
@@ -428,7 +434,7 @@ def new_document(source_path, settings=None):
             Runtime settings.  If none are provided, a default core set will
             be used.  If you will use the document object with any Docutils
             components, you must provide their default settings as well.  For
-            example, if parsing, at least provide the parser settings,
+            example, if parsing rST, at least provide the rst-parser settings,
             obtainable as follows::
 
                 settings = docutils.frontend.OptionParser(
@@ -546,9 +552,8 @@ def get_trim_footnote_ref_space(settings):
     If trim_footnote_reference_space is None, return False unless the
     footnote reference style is 'superscript'.
     """
-    if settings.trim_footnote_reference_space is None:
-        return hasattr(settings, 'footnote_references') and \
-               settings.footnote_references == 'superscript'
+    if settings.setdefault('trim_footnote_reference_space', None) is None:
+        return getattr(settings, 'footnote_references', None) == 'superscript'
     else:
         return settings.trim_footnote_reference_space
 
@@ -576,18 +581,7 @@ def escape2null(text):
         parts.append('\x00' + text[found+1:found+2])
         start = found + 2               # skip character after escape
 
-def unescape(text, restore_backslashes=False, respect_whitespace=False):
-    """
-    Return a string with nulls removed or restored to backslashes.
-    Backslash-escaped spaces are also removed.
-    """
-    # `respect_whitespace` is ignored (since introduction 2016-12-16)
-    if restore_backslashes:
-        return text.replace('\x00', '\\')
-    else:
-        for sep in ['\x00 ', '\x00\n', '\x00']:
-            text = ''.join(text.split(sep))
-        return text
+# `unescape` definition moved to `nodes` to avoid circular import dependency.
 
 def split_escaped_whitespace(text):
     """
@@ -600,7 +594,7 @@ def split_escaped_whitespace(text):
     return list(itertools.chain(*strings))
 
 def strip_combining_chars(text):
-    if isinstance(text, str) and sys.version_info < (3,0):
+    if isinstance(text, str) and sys.version_info < (3, 0):
         return text
     return u''.join([c for c in text if not unicodedata.combining(c)])
 
@@ -612,7 +606,7 @@ def find_combining_chars(text):
     [3, 6, 9]
 
     """
-    if isinstance(text, str) and sys.version_info < (3,0):
+    if isinstance(text, str) and sys.version_info < (3, 0):
         return []
     return [i for i,c in enumerate(text) if unicodedata.combining(c)]
 
@@ -626,7 +620,7 @@ def column_indices(text):
     """
     # TODO: account for asian wide chars here instead of using dummy
     # replacements in the tableparser?
-    string_indices = range(len(text))
+    string_indices = list(range(len(text)))
     for index in find_combining_chars(text):
         string_indices[index] = None
     return [i for i in string_indices if i is not None]
@@ -646,7 +640,7 @@ def column_width(text):
 
     Correct ``len(text)`` for wide East Asian and combining Unicode chars.
     """
-    if isinstance(text, str) and sys.version_info < (3,0):
+    if isinstance(text, str) and sys.version_info < (3, 0):
         return len(text)
     width = sum([east_asian_widths[unicodedata.east_asian_width(c)]
                  for c in text])
@@ -661,13 +655,6 @@ def uniq(L):
              r.append(item)
      return r
 
-def unique_combinations(items, n):
-    """Return `itertools.combinations`."""
-    warnings.warn('docutils.utils.unique_combinations is deprecated; '
-                  'use itertools.combinations directly.',
-                      DeprecationWarning, stacklevel=2)
-    return itertools.combinations(items, n)
-
 def normalize_language_tag(tag):
     """Return a list of normalized combinations for a `BCP 47` language tag.
 
@@ -681,7 +668,7 @@ def normalize_language_tag(tag):
 
     """
     # normalize:
-    tag = tag.lower().replace('-','_')
+    tag = tag.lower().replace('-', '_')
     # split (except singletons, which mark the following tag as non-standard):
     tag = re.sub(r'_([a-zA-Z0-9])_', r'_\1-', tag)
     subtags = [subtag for subtag in tag.split('_')]
@@ -689,7 +676,6 @@ def normalize_language_tag(tag):
     # find all combinations of subtags
     taglist = []
     for n in range(len(subtags), 0, -1):
-        # for tags in unique_combinations(subtags, n):
         for tags in itertools.combinations(subtags, n):
             taglist.append('-'.join(base_tag+tags))
     taglist += base_tag
