@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# $Id: manpage.py 8603 2021-01-08 15:24:32Z milde $
+# $Id: manpage.py 8860 2021-10-22 16:39:59Z milde $
 # Author: Engelbert Gruber <grubert@users.sourceforge.net>
 # Copyright: This module is put into the public domain.
 
@@ -216,7 +216,7 @@ class Translator(nodes.NodeVisitor):
         # Hopefully ``C`` courier too.
         self.defs = {
                 'indent': ('.INDENT %.1f\n', '.UNINDENT\n'),
-                'definition_list_item': ('.TP', ''),
+                'definition_list_item': ('.TP', ''), # paragraph with hanging tag
                 'field_name': ('.TP\n.B ', '\n'),
                 'literal': ('\\fB', '\\fP'),
                 'literal_block': ('.sp\n.nf\n.ft C\n', '\n.ft P\n.fi\n'),
@@ -290,6 +290,7 @@ class Translator(nodes.NodeVisitor):
             (u'\'', u'\\(aq'),
             (u'´', u"\\'"),
             (u'`', u'\\(ga'),
+            (u'"', u'\\(dq'),  # double quotes are a problem on macro lines
             ]
         for (in_char, out_markup) in replace_pairs:
             text = text.replace(in_char, out_markup)
@@ -385,10 +386,11 @@ class Translator(nodes.NodeVisitor):
         """append header with .TH and .SH NAME"""
         # NOTE before everything
         # .TH title_upper section date source manual
+        # BUT macros before .TH for whatis database generators.
         if self.header_written:
             return
-        self.head.append(self.header())
         self.head.append(MACRO_DEF)
+        self.head.append(self.header())
         self.header_written = 1
 
     def visit_address(self, node):
@@ -449,7 +451,7 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def visit_block_quote(self, node):
-        # BUG/HACK: indent always uses the _last_ indention,
+        # BUG/HACK: indent always uses the _last_ indentation,
         # thus we need two of them.
         self.indent(BLOCKQOUTE_INDENT)
         self.indent(0)
@@ -831,7 +833,7 @@ class Translator(nodes.NodeVisitor):
         self.body.append(self.defs['literal'][1])
 
     def visit_literal_block(self, node):
-        # BUG/HACK: indent always uses the _last_ indention,
+        # BUG/HACK: indent always uses the _last_ indentation,
         # thus we need two of them.
         self.indent(LITERAL_BLOCK_INDENT)
         self.indent(0)
@@ -901,7 +903,7 @@ class Translator(nodes.NodeVisitor):
         # options with parameter bold italic, .BI, -f file
         #
         # we do not know if .B or .BI
-        self.context.append('.B ')          # blind guess. Add blank for spinx see docutils/bugs/380
+        self.context.append('.B ')          # blind guess. Add blank for sphinx see docutils/bugs/380
         self.context.append(len(self.body)) # to be able to insert later
         self.context.append(0)              # option counter
 
