@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# $Id: test_nodes.py 7400 2012-05-01 08:34:41Z grubert $
+# $Id: test_nodes.py 8802 2021-07-13 21:52:22Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -11,18 +11,15 @@ Test module for nodes.py.
 
 import sys
 import unittest
-import types
+
 import DocutilsTestSupport              # must be imported before docutils
 from DocutilsTestSupport import nodes, utils
-from docutils._compat import b
 
 debug = False
 
-# python 2.3
-if not hasattr(unittest.TestCase, "assertTrue"):
-    # HACK? this changes TestCase, fixes the problem for tests executing afterwards.
-    # this tests break if run alone
-    unittest.TestCase.assertTrue = unittest.TestCase.failUnless
+if sys.version_info >= (3, 0):
+    unicode = str  # noqa
+
 
 class TextTests(unittest.TestCase):
 
@@ -38,6 +35,11 @@ class TextTests(unittest.TestCase):
         self.assertEqual(repr(self.text), r"<#text: 'Line 1.\nLine 2.'>")
         self.assertEqual(self.text.shortrepr(),
                           r"<#text: 'Line 1.\nLine 2.'>")
+        self.assertEqual(nodes.reprunicode('foo'), u'foo')
+        if sys.version_info < (3, 0):
+            self.assertEqual(repr(self.unicode_text), r"<#text: 'M\xf6hren'>")
+        else:
+            self.assertEqual(repr(self.unicode_text), u"<#text: 'Möhren'>")
 
     def test_str(self):
         self.assertEqual(str(self.text), 'Line 1.\nLine 2.')
@@ -47,18 +49,28 @@ class TextTests(unittest.TestCase):
         self.assertEqual(str(self.unicode_text), 'M\xf6hren')
 
     def test_astext(self):
-        self.assertEqual(self.text.astext(), 'Line 1.\nLine 2.')
+        self.assertTrue(isinstance(self.text.astext(), unicode))
+        self.assertEqual(self.text.astext(), u'Line 1.\nLine 2.')
+        self.assertEqual(self.unicode_text.astext(), u'Möhren')
 
     def test_pformat(self):
-        self.assertEqual(self.text.pformat(), 'Line 1.\nLine 2.\n')
+        self.assertTrue(isinstance(self.text.pformat(), unicode))
+        self.assertEqual(self.text.pformat(), u'Line 1.\nLine 2.\n')
+
+    def test_strip(self):
+        text = nodes.Text(' was noch ')
+        stripped = text.lstrip().rstrip()
+        stripped2 = text.lstrip(' wahn').rstrip(' wahn')
+        self.assertEqual(stripped, u'was noch')
+        self.assertEqual(stripped2, u's noc')
 
     def test_asciirestriction(self):
-        if sys.version_info < (3,):
+        if sys.version_info < (3, 0):
             self.assertRaises(UnicodeDecodeError, nodes.Text,
-                              b('hol%s' % chr(224)))
+                              b'hol%s' % chr(224))
         else:
             # no bytes at all allowed
-            self.assertRaises(TypeError, nodes.Text, b('hol'))
+            self.assertRaises(TypeError, nodes.Text, b'hol')
 
     def test_longrepr(self):
         self.assertEqual(repr(self.longtext), r"<#text: 'Mary had a "
@@ -86,18 +98,32 @@ class ElementTests(unittest.TestCase):
         del element['attr']
         element['mark'] = u'\u2022'
         self.assertEqual(repr(element), '<Element: >')
-        if sys.version_info < (3,):
+        if sys.version_info < (3, 0):
             self.assertEqual(str(element), '<Element mark="\\u2022"/>')
         else:
-            self.assertEqual(str(element), '<Element mark="\u2022"/>')
+            self.assertEqual(str(element), u'<Element mark="\u2022"/>')
         dom = element.asdom()
         self.assertEqual(dom.toxml(), u'<Element mark="\u2022"/>')
         dom.unlink()
+        element['names'] = ['nobody', u'имя', u'näs']
+        if sys.version_info < (3, 0):
+            self.assertEqual(repr(element),
+                '<Element "nobody; \\u0438\\u043c\\u044f; n\\xe4s": >')
+        else:
+            self.assertEqual(repr(element), u'<Element "nobody; имя; näs": >')
+        self.assertTrue(isinstance(repr(element), str))
 
     def test_withtext(self):
         element = nodes.Element('text\nmore', nodes.Text('text\nmore'))
+        uelement = nodes.Element(u'grün', nodes.Text(u'grün'))
         self.assertEqual(repr(element), r"<Element: <#text: 'text\nmore'>>")
+        if sys.version_info < (3, 0):
+            self.assertEqual(repr(uelement), "<Element: <#text: 'gr\\xfcn'>>")
+        else:
+            self.assertEqual(repr(uelement), u"<Element: <#text: 'grün'>>")
+        self.assertTrue(isinstance(repr(uelement), str))
         self.assertEqual(str(element), '<Element>text\nmore</Element>')
+        self.assertEqual(str(uelement), '<Element>gr\xfcn</Element>')
         dom = element.asdom()
         self.assertEqual(dom.toxml(), '<Element>text\nmore</Element>')
         dom.unlink()
@@ -149,6 +175,117 @@ class ElementTests(unittest.TestCase):
         # 'test' is not overwritten because it is not a basic attribute.
         self.assertEqual(element1['test'], ['test1'])
 
+    def test_update_all_atts(self):
+        # Note: Also tests is_not_list_attribute and is_not_known_attribute
+        # and various helpers
+        ## Test for full attribute replacement
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element2 = nodes.Element(ids=['baz', 'qux'], child_only='child',
+                                 all_nodes='dad', source='source')
+
+        # Test for when same fields are replaced as well as source...
+        element1.update_all_atts_consistantly(element2, True, True)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEqual(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEqual(element1['parent_only'], 'parent')
+        # 'all_nodes' is overwritten due to the second parameter == True.
+        self.assertEqual(element1['all_nodes'], 'dad')
+        # 'child_only' should have been added.
+        self.assertEqual(element1['child_only'], 'child')
+        # 'source' is also overwritten due to the third parameter == True.
+        self.assertEqual(element1['source'], 'source')
+
+        # Test for when same fields are replaced but not source...
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element1.update_all_atts_consistantly(element2)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEqual(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEqual(element1['parent_only'], 'parent')
+        # 'all_nodes' is overwritten due to the second parameter default of True.
+        self.assertEqual(element1['all_nodes'], 'dad')
+        # 'child_only' should have been added.
+        self.assertEqual(element1['child_only'], 'child')
+        # 'source' remains unset due to the third parameter default of False.
+        self.assertEqual(element1.get('source'), None)
+
+        # Test for when fields are NOT replaced but source is...
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element1.update_all_atts_consistantly(element2, False, True)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEqual(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEqual(element1['parent_only'], 'parent')
+        # 'all_nodes' is preserved due to the second parameter == False.
+        self.assertEqual(element1['all_nodes'], 'mom')
+        # 'child_only' should have been added.
+        self.assertEqual(element1['child_only'], 'child')
+        # 'source' is added due to the third parameter == True.
+        self.assertEqual(element1['source'], 'source')
+        element1 = nodes.Element(source='destination')
+        element1.update_all_atts_consistantly(element2, False, True)
+        # 'source' remains unchanged due to the second parameter == False.
+        self.assertEqual(element1['source'], 'destination')
+
+        # Test for when same fields are replaced but not source...
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element1.update_all_atts_consistantly(element2, False)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEqual(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEqual(element1['parent_only'], 'parent')
+        # 'all_nodes' is preserved due to the second parameter == False.
+        self.assertEqual(element1['all_nodes'], 'mom')
+        # 'child_only' should have been added.
+        self.assertEqual(element1['child_only'], 'child')
+        # 'source' remains unset due to the third parameter default of False.
+        self.assertEqual(element1.get('source'), None)
+
+        ## Test for List attribute merging
+        # Attribute Concatination
+        element1 = nodes.Element(ss='a', sl='1', ls=['I'], ll=['A'])
+        element2 = nodes.Element(ss='b', sl=['2'], ls='II', ll=['B'])
+        element1.update_all_atts_concatenating(element2)
+        # 'ss' is replaced because non-list
+        self.assertEqual(element1['ss'], 'b')
+        # 'sl' is replaced because they are both not lists
+        self.assertEqual(element1['sl'], ['2'])
+        # 'ls' is replaced because they are both not lists
+        self.assertEqual(element1['ls'], 'II')
+        # 'll' is extended because they are both lists
+        self.assertEqual(element1['ll'], ['A', 'B'])
+
+        # Attribute Coercion
+        element1 = nodes.Element(ss='a', sl='1', ls=['I'], ll=['A'])
+        element2 = nodes.Element(ss='b', sl=['2'], ls='II', ll=['B'])
+        element1.update_all_atts_coercion(element2)
+        # 'ss' is replaced because non-list
+        self.assertEqual(element1['ss'], 'b')
+        # 'sl' is converted to a list and appended because element2 has a list
+        self.assertEqual(element1['sl'], ['1', '2'])
+        # 'ls' has element2's value appended to the list
+        self.assertEqual(element1['ls'], ['I', 'II'])
+        # 'll' is extended because they are both lists
+        self.assertEqual(element1['ll'], ['A', 'B'])
+
+        # Attribute Conversion
+        element1 = nodes.Element(ss='a', sl='1', ls=['I'], ll=['A'])
+        element2 = nodes.Element(ss='b', sl=['2'], ls='II', ll=['B'])
+        element1.update_all_atts_convert(element2)
+        # 'ss' is converted to a list with the values from each element
+        self.assertEqual(element1['ss'], ['a', 'b'])
+        # 'sl' is converted to a list and appended
+        self.assertEqual(element1['sl'], ['1', '2'])
+        # 'ls' has element2's value appended to the list
+        self.assertEqual(element1['ls'], ['I', 'II'])
+        # 'll' is extended
+        self.assertEqual(element1['ll'], ['A', 'B'])
+
     def test_replace_self(self):
         parent = nodes.Element(ids=['parent'])
         child1 = nodes.Element(ids=['child1'])
@@ -193,17 +330,36 @@ class ElementTests(unittest.TestCase):
         self.assertEqual(len(parent), 5)
 
     def test_unicode(self):
-        node = nodes.Element(u'Möhren', nodes.Text(u'Möhren', u'Möhren'))
+        node = nodes.Element(u'Möhren', nodes.Text(u'Möhren'))
         self.assertEqual(unicode(node), u'<Element>Möhren</Element>')
 
 
 class MiscTests(unittest.TestCase):
 
+    def test_reprunicode(self):
+        # return `unicode` instance
+        self.assertTrue(isinstance(nodes.reprunicode('foo'), unicode))
+        self.assertEqual(nodes.reprunicode('foo'), u'foo')
+        self.assertEqual(nodes.reprunicode(u'Möhre'), u'Möhre')
+        if sys.version_info < (3, 0): # strip leading "u" from representation
+            self.assertEqual(repr(nodes.reprunicode(u'Möhre')),
+                             repr(u'Möhre')[1:])
+        else: # no change to `unicode` under Python 3k
+            self.assertEqual(repr(nodes.reprunicode(u'Möhre')), repr(u'Möhre'))
+
+    def test_ensure_str(self):
+        self.assertTrue(isinstance(nodes.ensure_str(u'über'), str))
+        self.assertEqual(nodes.ensure_str('over'), 'over')
+        if sys.version_info < (3, 0): # strip leading "u" from representation
+            self.assertEqual(nodes.ensure_str(u'über'), r'\xfcber')
+        else:
+            self.assertEqual(nodes.ensure_str(u'über'), r'über')
+
     def test_node_class_names(self):
         node_class_names = []
         for x in dir(nodes):
             c = getattr(nodes, x)
-            if isinstance(c, (type, types.ClassType)) and \
+            if isinstance(c, type) and \
                    issubclass(c, nodes.Node) and len(c.__bases__) > 1:
                 node_class_names.append(x)
         node_class_names.sort()
@@ -250,8 +406,6 @@ class MiscTests(unittest.TestCase):
             (u'\u024b q with hook tail', 'q-q-with-hook-tail'),
             (u'\u024d r with stroke', 'r-r-with-stroke'),
             (u'\u024f y with stroke', 'y-y-with-stroke'),
-          ]
-    ids_unicode_not_2_2 = [
            # From Latin-1 Supplements
             (u'\u00e0: a with grave', 'a-a-with-grave'),
             (u'\u00e1 a with acute', 'a-a-with-acute'),
@@ -384,9 +538,6 @@ class MiscTests(unittest.TestCase):
     def test_make_id(self):
         failures = []
         tests = self.ids + self.ids_unicode_all
-        import sys
-        if sys.version_info[:2] != (2, 2):
-            tests += self.ids_unicode_not_2_2
         for input, expect in tests:
             output = nodes.make_id(input)
             if expect != output:
@@ -456,16 +607,19 @@ class MiscTests(unittest.TestCase):
         return x not in self.testlist
 
     def test_copy(self):
-        grandchild = nodes.Text('rawsource')
-        child = nodes.emphasis('rawsource', grandchild, att='child')
-        e = nodes.Element('rawsource', child, att='e')
+        grandchild = nodes.Text('grandchild text')
+        child = nodes.emphasis('childtext', grandchild, att='child')
+        e = nodes.Element('raw text', child, att='e')
         # Shallow copy:
         e_copy = e.copy()
         self.assertTrue(e is not e_copy)
         # Internal attributes (like `rawsource`) are also copied.
-        self.assertEqual(e.rawsource, 'rawsource')
+        self.assertEqual(e.rawsource, 'raw text')
         self.assertEqual(e_copy.rawsource, e.rawsource)
         self.assertEqual(e_copy['att'], 'e')
+        self.assertEqual(e_copy.document, e.document)
+        self.assertEqual(e_copy.source, e.source)
+        self.assertEqual(e_copy.line, e.line)
         # Children are not copied.
         self.assertEqual(len(e_copy), 0)
         # Deep copy:
@@ -477,6 +631,16 @@ class MiscTests(unittest.TestCase):
         self.assertTrue(e_deepcopy[0][0] is not grandchild)
         self.assertEqual(e_deepcopy[0]['att'], 'child')
 
+    def test_system_message_copy(self):
+        e = nodes.system_message('mytext', att='e', rawsource='raw text')
+        # Shallow copy:
+        e_copy = e.copy()
+        self.assertTrue(e is not e_copy)
+        # Internal attributes (like `rawsource`) are also copied.
+        self.assertEqual(e.rawsource, 'raw text')
+        self.assertEqual(e_copy.rawsource, e.rawsource)
+        self.assertEqual(e_copy['att'], 'e')
+
 
 class TreeCopyVisitorTests(unittest.TestCase):
 
@@ -494,7 +658,7 @@ class TreeCopyVisitorTests(unittest.TestCase):
 
     def compare_trees(self, one, two):
         self.assertEqual(one.__class__, two.__class__)
-        self.assertNotEquals(id(one), id(two))
+        self.assertNotEqual(id(one), id(two))
         self.assertEqual(len(one.children), len(two.children))
         for i in range(len(one.children)):
             self.compare_trees(one.children[i], two.children[i])
@@ -507,6 +671,84 @@ class TreeCopyVisitorTests(unittest.TestCase):
         self.compare_trees(self.document, newtree)
 
 
+class SetIdTests(unittest.TestCase):
+
+    def setUp(self):
+        self.document = utils.new_document('test')
+        self.elements = [nodes.Element(names=['test']),
+                         nodes.section(), # Name empty
+                         nodes.section(names=['Test']), # duplicate id
+                         nodes.footnote(names=['2019-10-30']), # id empty
+                        ]
+
+    def test_set_id_default(self):
+        # Default prefixes.
+        for element in self.elements:
+            self.document.set_id(element)
+        ids = [element['ids'] for element in self.elements]
+        self.assertEqual(ids, [['test'], ['section-1'],
+                               ['test-1'], ['footnote-1']])
+
+    def test_set_id_custom(self):
+        # Custom prefixes.
+        
+        # Change settings.
+        self.document.settings.id_prefix = 'P-'
+        self.document.settings.auto_id_prefix = 'auto'
+
+        for element in self.elements:
+            self.document.set_id(element)
+        ids = [element['ids'] for element in self.elements]
+        self.assertEqual(ids, [['P-test'],
+                               ['P-auto1'],
+                               ['P-auto2'],
+                               ['P-2019-10-30']])
+
+    def test_set_id_descriptive_auto_id(self):
+        # Use name or tag-name for auto-id.
+        
+        # Change setting.
+        self.document.settings.auto_id_prefix = '%'
+
+        for element in self.elements:
+            self.document.set_id(element)
+        ids = [element['ids'] for element in self.elements]
+        self.assertEqual(ids, [['test'],
+                               ['section-1'],
+                               ['test-1'],
+                               ['footnote-1']])
+
+    def test_set_id_custom_descriptive_auto_id(self):
+        # Custom prefixes and name or tag-name for auto-id.
+        
+        # Change settings.
+        self.document.settings.id_prefix = 'P:'
+        self.document.settings.auto_id_prefix = 'a-%'
+
+        for element in self.elements:
+            self.document.set_id(element)
+        ids = [element['ids'] for element in self.elements]
+        self.assertEqual(ids, [['P:test'],
+                               ['P:a-section-1'],
+                               ['P:test-1'],
+                               ['P:2019-10-30']])
+
+class NodeVisitorTests(unittest.TestCase):
+    def setUp(self):
+        self.document = utils.new_document('test')
+        self.element = nodes.Element()
+        self.visitor = nodes.NodeVisitor(self.document)
+
+    def test_dispatch_visit_unknown(self):
+        # raise exception if no visit/depart methods are defined for node class
+        self.assertRaises(NotImplementedError, 
+                          self.visitor.dispatch_visit, self.element)
+
+    def test_dispatch_visit_optional(self):
+        # silently skip nodes of a calss in tuple nodes.NodeVisitor.optional
+        rv = self.visitor.dispatch_visit(nodes.meta())
+        self.assertIsNone(rv)
+
 class MiscFunctionTests(unittest.TestCase):
 
     names = [('a', 'a'), ('A', 'a'), ('A a A', 'a a a'),
@@ -518,33 +760,6 @@ class MiscFunctionTests(unittest.TestCase):
             normed = nodes.fully_normalize_name(input)
             self.assertEqual(normed, output)
 
-    def test_set_id_default(self):
-        # Default prefixes.
-        document = utils.new_document('test')
-        # From name.
-        element = nodes.Element(names=['test'])
-        document.set_id(element)
-        self.assertEqual(element['ids'], ['test'])
-        # Auto-generated.
-        element = nodes.Element()
-        document.set_id(element)
-        self.assertEqual(element['ids'], ['id1'])
-
-    def test_set_id_custom(self):
-        # Custom prefixes.
-        document = utils.new_document('test')
-        # Change settings.
-        document.settings.id_prefix = 'prefix'
-        document.settings.auto_id_prefix = 'auto'
-        # From name.
-        element = nodes.Element(names=['test'])
-        document.set_id(element)
-        self.assertEqual(element['ids'], ['prefixtest'])
-        # Auto-generated.
-        element = nodes.Element()
-        document.set_id(element)
-        self.assertEqual(element['ids'], ['prefixauto1'])
-        
 
 if __name__ == '__main__':
     unittest.main()
