@@ -1,16 +1,19 @@
+
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3129 $
-# Date: $Date: 2005-03-26 17:21:28 +0100 (Sat, 26 Mar 2005) $
+# $Id: test_targets.py 8771 2021-06-18 18:55:08Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for states.py.
 """
+from __future__ import absolute_import
+
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 
-from __init__ import DocutilsTestSupport
 
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
@@ -36,9 +39,9 @@ totest['targets'] = [
 """,
 """\
 <document source="test data">
-    <target ids="optional-space-before-colon" names="optional space before colon">
+    <target ids="optional-space-before-colon" names="optional\\ space\\ before\\ colon">
 """],
-["""\
+[r"""
 External hyperlink targets:
 
 .. _one-liner: http://structuredtext.sourceforge.net
@@ -51,7 +54,10 @@ External hyperlink targets:
    http://structuredtext.
    sourceforge.net
 
-.. _not-indirect: uri\\_
+.. _escaped-whitespace: http://example.org/a\ path\ with\
+   spaces.html
+
+.. _not-indirect: uri\_
 """,
 """\
 <document source="test data">
@@ -60,6 +66,7 @@ External hyperlink targets:
     <target ids="one-liner" names="one-liner" refuri="http://structuredtext.sourceforge.net">
     <target ids="starts-on-this-line" names="starts-on-this-line" refuri="http://structuredtext.sourceforge.net">
     <target ids="entirely-below" names="entirely-below" refuri="http://structuredtext.sourceforge.net">
+    <target ids="escaped-whitespace" names="escaped-whitespace" refuri="http://example.org/a path with spaces.html">
     <target ids="not-indirect" names="not-indirect" refuri="uri_">
 """],
 ["""\
@@ -81,13 +88,30 @@ Indirect hyperlink targets:
 
 .. _`a target name: including a colon (quoted)`:
 
-.. _a target name\: including a colon (escaped):
+.. _a target name\\: including a colon (escaped):
 """,
 """\
 <document source="test data">
-    <target ids="a-long-target-name" names="a long target name">
-    <target ids="a-target-name-including-a-colon-quoted" names="a target name: including a colon (quoted)">
-    <target ids="a-target-name-including-a-colon-escaped" names="a target name: including a colon (escaped)">
+    <target ids="a-long-target-name" names="a\\ long\\ target\\ name">
+    <target ids="a-target-name-including-a-colon-quoted" names="a\\ target\\ name:\\ including\\ a\\ colon\\ (quoted)">
+    <target ids="a-target-name-including-a-colon-escaped" names="a\\ target\\ name:\\ including\\ a\\ colon\\ (escaped)">
+"""],
+["""\
+.. _`target: No matching backquote.
+.. _`: No matching backquote either.
+""",
+"""\
+<document source="test data">
+    <comment xml:space="preserve">
+        _`target: No matching backquote.
+    <system_message level="2" line="1" source="test data" type="WARNING">
+        <paragraph>
+            malformed hyperlink target.
+    <comment xml:space="preserve">
+        _`: No matching backquote either.
+    <system_message level="2" line="2" source="test data" type="WARNING">
+        <paragraph>
+            malformed hyperlink target.
 """],
 ["""\
 .. _a very long target name,
@@ -97,8 +121,8 @@ Indirect hyperlink targets:
 """,
 """\
 <document source="test data">
-    <target ids="a-very-long-target-name-split-across-lines" names="a very long target name, split across lines">
-    <target ids="and-another-with-backquotes" names="and another, with backquotes">
+    <target ids="a-very-long-target-name-split-across-lines" names="a\\ very\\ long\\ target\\ name,\\ split\\ across\\ lines">
+    <target ids="and-another-with-backquotes" names="and\\ another,\\ with\\ backquotes">
 """],
 ["""\
 External hyperlink:
@@ -120,7 +144,29 @@ External hyperlink:
 """\
 <document source="test data">
     <target ids="email" names="email" refuri="mailto:jdoe@example.com">
-    <target ids="multi-line-email" names="multi-line email" refuri="mailto:jdoe@example.com">
+    <target ids="multi-line-email" names="multi-line\\ email" refuri="mailto:jdoe@example.com">
+"""],
+["""\
+Malformed target:
+
+.. __malformed: no good
+
+Target beginning with an underscore:
+
+.. _`_target`: OK
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Malformed target:
+    <comment xml:space="preserve">
+        __malformed: no good
+    <system_message level="2" line="3" source="test data" type="WARNING">
+        <paragraph>
+            malformed hyperlink target.
+    <paragraph>
+        Target beginning with an underscore:
+    <target ids="target" names="_target" refuri="OK">
 """],
 ["""\
 Duplicate external targets (different URIs):
@@ -134,10 +180,10 @@ Duplicate external targets (different URIs):
     <paragraph>
         Duplicate external targets (different URIs):
     <target dupnames="target" ids="target" refuri="first">
-    <system_message backrefs="id1" level="2" line="5" source="test data" type="WARNING">
+    <system_message backrefs="target-1" level="2" line="5" source="test data" type="WARNING">
         <paragraph>
             Duplicate explicit target name: "target".
-    <target dupnames="target" ids="id1" refuri="second">
+    <target dupnames="target" ids="target-1" refuri="second">
 """],
 ["""\
 Duplicate external targets (same URIs):
@@ -151,10 +197,10 @@ Duplicate external targets (same URIs):
     <paragraph>
         Duplicate external targets (same URIs):
     <target ids="target" names="target" refuri="first">
-    <system_message backrefs="id1" level="1" line="5" source="test data" type="INFO">
+    <system_message backrefs="target-1" level="1" line="5" source="test data" type="INFO">
         <paragraph>
             Duplicate explicit target name: "target".
-    <target dupnames="target" ids="id1" refuri="first">
+    <target dupnames="target" ids="target-1" refuri="first">
 """],
 ["""\
 Duplicate implicit targets.
@@ -178,10 +224,10 @@ Paragraph.
             Title
         <paragraph>
             Paragraph.
-    <section dupnames="title" ids="id1">
+    <section dupnames="title" ids="title-1">
         <title>
             Title
-        <system_message backrefs="id1" level="1" line="9" source="test data" type="INFO">
+        <system_message backrefs="title-1" level="1" line="9" source="test data" type="INFO">
             <paragraph>
                 Duplicate implicit target name: "title".
         <paragraph>
@@ -204,14 +250,38 @@ Paragraph.
     <section dupnames="title" ids="title">
         <title>
             Title
-        <system_message backrefs="id1" level="1" line="6" source="test data" type="INFO">
+        <system_message backrefs="title-1" level="1" line="6" source="test data" type="INFO">
             <paragraph>
                 Duplicate implicit target name: "title".
-        <target ids="id1" names="title">
+        <target ids="title-1" names="title">
         <paragraph>
             Paragraph.
 """],
 ["""\
+Duplicate implicit/directive targets.
+
+Title
+=====
+
+.. target-notes::
+   :name: title
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Duplicate implicit/directive targets.
+    <section dupnames="title" ids="title">
+        <title>
+            Title
+        <pending ids="title-1" names="title">
+            <system_message backrefs="title-1" level="1" line="4" source="test data" type="INFO">
+                <paragraph>
+                    Duplicate implicit target name: "title".
+            .. internal attributes:
+                 .transform: docutils.transforms.references.TargetNotes
+                 .details:
+"""],
+["""\
 Duplicate explicit targets.
 
 .. _title:
@@ -233,20 +303,44 @@ Third.
     <target dupnames="title" ids="title">
     <paragraph>
         First.
-    <system_message backrefs="id1" level="2" line="7" source="test data" type="WARNING">
+    <system_message backrefs="title-1" level="2" line="7" source="test data" type="WARNING">
         <paragraph>
             Duplicate explicit target name: "title".
-    <target dupnames="title" ids="id1">
+    <target dupnames="title" ids="title-1">
     <paragraph>
         Second.
-    <system_message backrefs="id2" level="2" line="11" source="test data" type="WARNING">
+    <system_message backrefs="title-2" level="2" line="11" source="test data" type="WARNING">
         <paragraph>
             Duplicate explicit target name: "title".
-    <target dupnames="title" ids="id2">
+    <target dupnames="title" ids="title-2">
     <paragraph>
         Third.
 """],
 ["""\
+Duplicate explicit/directive targets.
+
+.. _title:
+
+First.
+
+.. rubric:: this is a title too
+   :name: title
+
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Duplicate explicit/directive targets.
+    <target dupnames="title" ids="title">
+    <paragraph>
+        First.
+    <rubric dupnames="title" ids="title-1">
+        this is a title too
+        <system_message backrefs="title-1" level="2" line="9" source="test data" type="WARNING">
+            <paragraph>
+                Duplicate explicit target name: "title".
+"""],
+["""\
 Duplicate targets:
 
 Target
@@ -254,7 +348,7 @@ Target
 
 Implicit section header target.
 
-.. [target] Citation target.
+.. [TARGET] Citation target.
 
 .. [#target] Autonumber-labeled footnote target.
 
@@ -263,6 +357,9 @@ Implicit section header target.
 Explicit internal target.
 
 .. _target: Explicit_external_target
+
+.. rubric:: directive with target
+   :name: Target
 """,
 """\
 <document source="test data">
@@ -273,30 +370,59 @@ Explicit internal target.
             Target
         <paragraph>
             Implicit section header target.
-        <citation dupnames="target" ids="id1">
+        <citation dupnames="target" ids="target-1">
             <label>
-                target
-            <system_message backrefs="id1" level="1" line="8" source="test data" type="INFO">
+                TARGET
+            <system_message backrefs="target-1" level="1" line="8" source="test data" type="INFO">
                 <paragraph>
                     Duplicate implicit target name: "target".
             <paragraph>
                 Citation target.
-        <footnote auto="1" dupnames="target" ids="id2">
-            <system_message backrefs="id2" level="2" line="10" source="test data" type="WARNING">
+        <footnote auto="1" dupnames="target" ids="target-2">
+            <system_message backrefs="target-2" level="2" line="10" source="test data" type="WARNING">
                 <paragraph>
                     Duplicate explicit target name: "target".
             <paragraph>
                 Autonumber-labeled footnote target.
-        <system_message backrefs="id3" level="2" line="12" source="test data" type="WARNING">
+        <system_message backrefs="target-3" level="2" line="12" source="test data" type="WARNING">
             <paragraph>
                 Duplicate explicit target name: "target".
-        <target dupnames="target" ids="id3">
+        <target dupnames="target" ids="target-3">
         <paragraph>
             Explicit internal target.
-        <system_message backrefs="id4" level="2" line="16" source="test data" type="WARNING">
+        <system_message backrefs="target-4" level="2" line="16" source="test data" type="WARNING">
             <paragraph>
                 Duplicate explicit target name: "target".
-        <target dupnames="target" ids="id4" refuri="Explicit_external_target">
+        <target dupnames="target" ids="target-4" refuri="Explicit_external_target">
+        <rubric dupnames="target" ids="target-5">
+            directive with target
+            <system_message backrefs="target-5" level="2" line="4" source="test data" type="WARNING">
+                <paragraph>
+                    Duplicate explicit target name: "target".
+"""],
+["""\
+.. _unescaped colon at end:: no good
+
+.. _:: no good either
+
+.. _escaped colon\\:: OK
+
+.. _`unescaped colon, quoted:`: OK
+""",
+"""\
+<document source="test data">
+    <comment xml:space="preserve">
+        _unescaped colon at end:: no good
+    <system_message level="2" line="1" source="test data" type="WARNING">
+        <paragraph>
+            malformed hyperlink target.
+    <comment xml:space="preserve">
+        _:: no good either
+    <system_message level="2" line="3" source="test data" type="WARNING">
+        <paragraph>
+            malformed hyperlink target.
+    <target ids="escaped-colon" names="escaped\\ colon:" refuri="OK">
+    <target ids="unescaped-colon-quoted" names="unescaped\\ colon,\\ quoted:" refuri="OK">
 """],
 ]
 
@@ -310,7 +436,7 @@ Anonymous external hyperlink target:
 <document source="test data">
     <paragraph>
         Anonymous external hyperlink target:
-    <target anonymous="1" ids="id1" refuri="http://w3c.org/">
+    <target anonymous="1" ids="target-1" refuri="http://w3c.org/">
 """],
 ["""\
 Anonymous external hyperlink target:
@@ -321,7 +447,7 @@ __ http://w3c.org/
 <document source="test data">
     <paragraph>
         Anonymous external hyperlink target:
-    <target anonymous="1" ids="id1" refuri="http://w3c.org/">
+    <target anonymous="1" ids="target-1" refuri="http://w3c.org/">
 """],
 ["""\
 Anonymous indirect hyperlink target:
@@ -332,7 +458,7 @@ Anonymous indirect hyperlink target:
 <document source="test data">
     <paragraph>
         Anonymous indirect hyperlink target:
-    <target anonymous="1" ids="id1" refname="reference">
+    <target anonymous="1" ids="target-1" refname="reference">
 """],
 ["""\
 Anonymous external hyperlink target, not indirect:
@@ -345,8 +471,8 @@ __ this URI ends with an underscore_
 <document source="test data">
     <paragraph>
         Anonymous external hyperlink target, not indirect:
-    <target anonymous="1" ids="id1" refuri="uri_">
-    <target anonymous="1" ids="id2" refuri="thisURIendswithanunderscore_">
+    <target anonymous="1" ids="target-1" refuri="uri_">
+    <target anonymous="1" ids="target-2" refuri="thisURIendswithanunderscore_">
 """],
 ["""\
 Anonymous indirect hyperlink targets:
@@ -359,8 +485,8 @@ __ `a very long
 <document source="test data">
     <paragraph>
         Anonymous indirect hyperlink targets:
-    <target anonymous="1" ids="id1" refname="reference">
-    <target anonymous="1" ids="id2" refname="a very long reference">
+    <target anonymous="1" ids="target-1" refname="reference">
+    <target anonymous="1" ids="target-2" refname="a very long reference">
 """],
 ["""\
 Mixed anonymous & named indirect hyperlink targets:
@@ -381,9 +507,9 @@ no blank line
 <document source="test data">
     <paragraph>
         Mixed anonymous & named indirect hyperlink targets:
-    <target anonymous="1" ids="id1" refname="reference">
-    <target anonymous="1" ids="id2" refname="reference">
-    <target anonymous="1" ids="id3" refname="reference">
+    <target anonymous="1" ids="target-1" refname="reference">
+    <target anonymous="1" ids="target-2" refname="reference">
+    <target anonymous="1" ids="target-3" refname="reference">
     <target ids="target1" names="target1" refname="reference">
     <system_message level="2" line="7" source="test data" type="WARNING">
         <paragraph>
@@ -391,9 +517,9 @@ no blank line
     <paragraph>
         no blank line
     <target ids="target2" names="target2" refname="reference">
-    <target anonymous="1" ids="id4" refname="reference">
-    <target anonymous="1" ids="id5" refname="reference">
-    <target anonymous="1" ids="id6" refname="reference">
+    <target anonymous="1" ids="target-4" refname="reference">
+    <target anonymous="1" ids="target-5" refname="reference">
+    <target anonymous="1" ids="target-6" refname="reference">
     <system_message level="2" line="13" source="test data" type="WARNING">
         <paragraph>
             Explicit markup ends without a blank line; unexpected unindent.
